@@ -18,6 +18,12 @@
 #include "textwindow.h"
 
 #include <hildon-mime.h>
+#include <dbus/dbus.h>
+
+#include <qlibrary.h>
+#include <stdlib.h>
+#include <qfile.h>
+#include <qurl.h>
 
 //#define QT_NO_KEYWORDS
 //#include <libosso-abook/osso-abook-contact.h>
@@ -46,7 +52,20 @@ void QrSink::clickAction(QWidget *parentWindow) {
         openInstallFile(barcodeText);
 
     }else if(type==QrSink::URL){
-        openBrowser(barcodeText);
+        // make sure our URL starts with http:// and ends at or before a space.
+        // may need to adjust this if e.g. a URL:/URLTO: type format is used without the http://
+        int url_start = barcodeText.indexOf("http://",0,Qt::CaseInsensitive);
+        QString trimmed_text;
+        if(url_start>0)
+            trimmed_text = barcodeText.mid(url_start,-1);
+        else
+            trimmed_text = barcodeText;
+
+        int url_end = trimmed_text.indexOf(" ",0,Qt::CaseInsensitive);
+        if (url_end>0)
+            trimmed_text = trimmed_text.mid(0,url_end-1);
+
+        openBrowser(trimmed_text);
 
     }else if(type==QrSink::iMode){
     }else if(type==QrSink::Email){
@@ -59,11 +78,12 @@ void QrSink::clickAction(QWidget *parentWindow) {
         if(!barcodeText.contains("SMS://",Qt::CaseInsensitive)){
             // change the uri from SMS: / SMSTO: to SMS://
             QString t(barcodeText);
-            if(barcodeText.contains("SMS:",Qt::CaseInsensitive)){
-                t.replace("SMS:", "SMS://", Qt::CaseInsensitive);
-            }else if(barcodeText.contains("SMSTO:",Qt::CaseInsensitive)){
-                t.replace("SMSTO:", "SMS://", Qt::CaseInsensitive);
+            if(barcodeText.contains("SMSTO:",Qt::CaseInsensitive)){
+                t.replace("SMSTO:", "SMS:", Qt::CaseInsensitive);
             }
+//            if(barcodeText.contains("SMS:",Qt::CaseInsensitive)){
+//                t.replace("SMS:", "SMS://", Qt::CaseInsensitive);
+//            }else
             openGeneric(t);
         }else
             openGeneric(barcodeText);
@@ -376,22 +396,99 @@ void QrSink::openInstallFile(QString data)
 }
 
 
+
+
 void QrSink::openGeneric(QString uri)
 {
 
     qDebug() << "In openGeneric()";
 
-    GError *error; // see if we can get away with this...
-    //HildonURIAction* action = hildon_uri_get_default_action_by_uri((const gchar*)uri.constData(), &error);
-    //hildon_uri_open((const gchar*)uri.constData(), action, &error);
+    //GError *error; // see if we can get away with this...
+    ////HildonURIAction* action = hildon_uri_get_default_action_by_uri((const gchar*)uri.constData(), &error);
+    ////hildon_uri_open((const gchar*)uri.constData(), action, &error);
 
     qDebug() << "openGeneric(): Trying to handle: " << uri.toAscii();
 
-    hildon_uri_open((const gchar*)uri.constData(), NULL, NULL);
+    //hildon_uri_open((const gchar*)uri.constData(), NULL, NULL);
+
+    ////hildon_uri_open ((const gchar*)uri.constData(), HILDON_URI_ACTION_NORMAL, &error);
+
+    QFileInfo file(uri);
+    if (file.exists()){
+        // we're a file!
+        qDebug() << "We're a file, using hildon_mime_open_file()";
+        DBusConnection* conn;
+        conn = dbus_bus_get(DBUS_BUS_SESSION, 0);
+        hildon_mime_open_file(conn, QUrl::fromLocalFile(file.absoluteFilePath()).toEncoded().constData());
+    }else{
+        // we're something else.... see if this will work then :(
+        qDebug() << "We're NOT a file, using QDesktopServices::openUrl()";
+        QDesktopServices::openUrl(QUrl(uri, QUrl::TolerantMode));
+    }
+
+
+
+    //
+
+
+
+
+
+
+//    QUrl url(uri, QUrl::TolerantMode);
+//    struct DBusConnection;
+//    typedef DBusConnection *(*Ptr_get_raw_handler_from_session_bus)();
+//    static Ptr_get_raw_handler_from_session_bus get_raw_handler_from_session_bus = 0;
+//    typedef void (*Ptr_hildon_mime_open)(DBusConnection *, const char *);
+//    static Ptr_hildon_mime_open hildon_mime_open = 0;
+//    typedef bool (*Ptr_hildon_uri_open)(const char *, void *, void **);
+//    static Ptr_hildon_uri_open hildon_uri_open = 0;
+//
+//    if (!get_raw_handler_from_session_bus) {
+//            get_raw_handler_from_session_bus = (Ptr_get_raw_handler_from_session_bus)QLibrary::resolve(QLatin1String("libQtDBus"), 4, "get_raw_handler_from_session_bus");
+//        }
+//
+//        DBusConnection* conn = 0;
+//        if (get_raw_handler_from_session_bus){
+//            conn = get_raw_handler_from_session_bus();
+//            if(!conn)
+//                qDebug()<<"error in getting dbus connection";
+//        }
+//        if (!hildon_mime_open && !hildon_uri_open) {
+//            QLibrary lib(QLatin1String("libhildonmime"), 0, 0);
+//            hildon_mime_open = (Ptr_hildon_mime_open)lib.resolve("hildon_mime_open_file");
+//            hildon_uri_open = (Ptr_hildon_uri_open)lib.resolve("hildon_uri_open");
+//        }
+//        QByteArray array = url.toEncoded();
+//        if(array.startsWith("file://")) {
+//            if (hildon_mime_open)
+//                hildon_mime_open(conn, array.constData());
+//            else
+//                qDebug()<<"error in hildon_mime_open_file";
+//        } else {
+//
+//            if (hildon_uri_open)
+//                hildon_uri_open(array.constData(), 0, 0);
+//            else
+//                qDebug()<<"error in hildon_open_uri";
+//
+//        }
+
+
+
+
+
+//    typedef bool (*Ptr_hildon_uri_open)(const char *, void *, void **);
+//    static Ptr_hildon_uri_open hildon_uri_open = 0;
+//
+//    if (!hildon_uri_open) {
+//        QLibrary lib(QLatin1String("libhildonmime"), 0, 0);
+//    hildon_uri_open = (Ptr_hildon_uri_open)lib.resolve("hildon_uri_open");
+//    }
+//    if (hildon_uri_open)
+//        hildon_uri_open((const char*)uri.constData(), 0, 0);
 
-    //hildon_uri_open ((const gchar*)uri.constData(), HILDON_URI_ACTION_NORMAL, &error);
 
-    //QDesktopServices::openUrl(uri);
 
 //#ifdef Q_WS_MAEMO_5 // Uses native file opening method
 //            //TODO: find better solution for this, maybe get fixed in Qt
@@ -436,7 +533,8 @@ void QrSink::addVCard(QString user_data)
 
     file.setAutoRemove(false); // we'll do this at the end
 
-    openGeneric(QString("file://" + file.fileName()));
+    //openGeneric(QString("file://" + file.fileName()));
+    openGeneric(file.fileName());
 
 
     //file.~QTemporaryFile();
